/********************************************************************
 * kreg3.c        24-DEC-1991             Y.Takeuchi                *
 *                                                                  *
 * Show K2917 registers (interrupt)                                 *
 * (SPARC  SFVME-100  K2917  K3922)                                 *
 ********************************************************************/
#include <stdio.h>
#include <errno.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/mman.h>
#include <sbusdev/sfvmereg.h>

#define BASE         0x0000FF00     /* K2917 base address */

#define WIN_ADDR     0x00000000     /* VME base address of window */
#define VME_SIZE     0x00010000     /* 64 Kbytes (Short I/O) */


#define W   unsigned short
#define B   unsigned char

struct K_REG {
    W cser;
    W dummy02;
    W docr;
    W sccr;
    W dummy05;
    W mtc;
    W machi;
    W maclo;

    W dummy11;           /* 0x10 - 0x1F */
    W dummy12;
    W dummy13;
    W dummy14;
    W dummy15;
    W dummy16;
    W dummy17;
    W dummy18;

    W dummy21;           /* 0x20 - 0x2F */
    W dummy22;
    W dummy23;
    W dummy24;
    W dummy25;
    W dummy26;
    W dummy27;
    W dummy28;

    W dummy31;           /* 0x30 - 0x3F */
    W dummy32;
    W dummy33;
    W dummy34;
    W dummy35;
    W dummy36;
    W dummy37;
    W dummy38;

    W lamc;
    W donc;
    W empc;
    W aboc;
    W lamv;
    W donv;
    W empv;
    W abov;

    W dummy51;           /* 0x50 - 0x5F */
    W dummy52;
    W dummy53;
    W dummy54;
    W dummy55;
    W dummy56;
    W dummy57;
    W dummy58;

    W amr;
    W cmr;
    W cma;
    W cwc;
    W srr;
    W dlr;
    W dhr;
    W csr;
};


main()
{
    int	    vme_fd;                 /* file descriptor for VME device */
    caddr_t vme_vaddr;              /* virtual address */
    int	    vme_waddr;              /* VME base address of window */
    struct  K_REG *k;

    vme_waddr = BASE;

/*********************************************************************
 * Open VME A16D16 Space for mapping                                 *
 *********************************************************************/
    if ((vme_fd = open("/dev/sfvme16D16", O_RDWR)) == -1)
        perror("File open: ");

/*********************************************************************
 * Set the base address of a VME window.                             *
 *********************************************************************/
    if (ioctl(vme_fd, SFVME_SET, &vme_waddr) == -1)
        perror("SFVME set: ");

/*********************************************************************
 * Map a portion of VME memory into the program's address space.     *
 *********************************************************************/
    vme_vaddr = mmap((caddr_t)0, VME_SIZE, PROT_READ | PROT_WRITE,
                     MAP_SHARED, vme_fd, WIN_ADDR);
    if (vme_vaddr == (caddr_t)-1) {
        perror("Mapping: ");
        exit(1);
    }

/*********************************************************************
 * main routine                                                      *
 *********************************************************************/

    k = (struct K_REG *)(vme_vaddr + BASE);

    for(;;) {
	printf("\n");
	printf("k->lamc     = %04X\n", k->lamc);
	printf("k->donc     = %04X\n", k->donc);
	printf("k->empc     = %04X\n", k->empc);
	printf("k->aboc     = %04X\n", k->aboc);
	printf("k->lamv     = %04X\n", k->lamv);
	printf("k->donv     = %04X\n", k->donv);
	printf("k->empv     = %04X\n", k->empv);
	printf("k->abov     = %04X\n", k->abov);
	printf("\n");
	sleep(1);
    }
}
